@@ -1,10 +1,11 @@
-import os
 from typing import List, Dict, Any
 
 from pydantic import BaseModel
 from openai import AsyncOpenAI
 from openai.types.chat.chat_completion_message import ChatCompletionMessage
 
+from academia_mcp.settings import settings
+
 
 class ChatMessage(BaseModel):  # type: ignore
     role: str
@@ -15,9 +16,9 @@ ChatMessages = List[ChatMessage]
 
 
 async def llm_acall(model_name: str, messages: ChatMessages, **kwargs: Any) -> str:
-    key = os.getenv("OPENROUTER_API_KEY", "")
+    key = settings.OPENROUTER_API_KEY
     assert key, "Please set OPENROUTER_API_KEY in the environment variables"
-    base_url = os.getenv("BASE_URL", "https://openrouter.ai/api/v1")
+    base_url = settings.BASE_URL
 
     client = AsyncOpenAI(base_url=base_url, api_key=key)
     response: ChatCompletionMessage = (
