@@ -1,12 +1,11 @@
 import re
-import json
-from typing import Optional, Dict, Any, cast
+from typing import Any, Dict, List, Optional
 
 from markdownify import markdownify  # type: ignore
+from pydantic import BaseModel, Field
 
-from academia_mcp.utils import get_with_retries, post_with_retries
 from academia_mcp.settings import settings
-from academia_mcp.utils import sanitize_output
+from academia_mcp.utils import get_with_retries, post_with_retries, sanitize_output
 
 EXA_CONTENTS_URL = "https://api.exa.ai/contents"
 TAVILY_EXTRACT_URL = "https://api.tavily.com/extract"
@@ -14,6 +13,16 @@ AVAILABLE_PROVIDERS = ("basic", "exa", "tavily")
 ERROR_MESSAGE = "Failed to get content from the page. Try to use another provider."
 
 
+class VisitWebpageResponse(BaseModel):  # type: ignore
+    id: str = Field(description="ID of the webpage, usually the URL")
+    provider: str = Field(description="Provider used to get the content")
+    text: Optional[str] = Field(description="Text content of the webpage", default=None)
+    images: List[str] = Field(description="Images of the webpage", default_factory=list)
+    error: Optional[str] = Field(
+        description="Error message if the webpage is not found", default=None
+    )
+
+
 def _exa_visit_webpage(url: str) -> Dict[str, Any]:
     key = settings.EXA_API_KEY or ""
     assert key, "Error: EXA_API_KEY is not set and no api_key was provided"
@@ -25,7 +34,7 @@ def _exa_visit_webpage(url: str) -> Dict[str, Any]:
     results = response.json()["results"]
     if not results:
         return {"error": ERROR_MESSAGE}
-    return cast(Dict[str, Any], results[0])
+    return {"text": results[0]["text"]}
 
 
 def _tavily_visit_webpage(url: str) -> Dict[str, Any]:
@@ -33,12 +42,15 @@ def _tavily_visit_webpage(url: str) -> Dict[str, Any]:
     assert key, "Error: TAVILY_API_KEY is not set and no api_key was provided"
     payload = {
         "urls": [url],
+        "extract_depth": "advanced",
+        "include_images": True,
     }
     response = post_with_retries(TAVILY_EXTRACT_URL, payload=payload, api_key=key)
     results = response.json()["results"]
     if not results:
         return {"error": ERROR_MESSAGE}
-    return {"text": results[0]["raw_content"]}
+    result = results[0]
+    return {"text": result["raw_content"], "images": result["images"]}
 
 
 def _basic_visit_webpage(url: str) -> Dict[str, Any]:
@@ -58,13 +70,9 @@ def _basic_visit_webpage(url: str) -> Dict[str, Any]:
         return {"error": str(e) + "\n" + ERROR_MESSAGE}
 
 
-def visit_webpage(url: str, provider: Optional[str] = "basic") -> str:
+def visit_webpage(url: str, provider: Optional[str] = "basic") -> VisitWebpageResponse:
     """
     Visit a webpage and return the content.
-
-    Returns a JSON object serialized to a string. The structure is: {"id": "...", "text": "..."}.
-    If there are errors, the structure is: {"id": "...", "error": "..."}.
-    Use `json.loads` to deserialize the result if you want to get specific fields.
     Try to use both "tavily" and "basic" providers. They might work differently for the same URL.
 
     Args:
@@ -82,6 +90,9 @@ def visit_webpage(url: str, provider: Optional[str] = "basic") -> str:
     else:
         result = _basic_visit_webpage(url)
 
-    result["id"] = url
-    result["provider"] = provider
-    return sanitize_output(json.dumps(result, ensure_ascii=False))
+    result = VisitWebpageResponse(id=url, provider=provider, **result)
+    if result.text:
+        result.text = sanitize_output(result.text)
+    if result.error:
+        result.error = sanitize_output(result.error)
+    return result
